import os
import re

INDIR = 'source/export'
OUTDIR = 'assets/mesh-animations/'


def build_material(infile, outfile):
    """Rewrite the material file so as to eliminate broken absolute paths."""
    with open(os.path.join(OUTDIR, outfile), 'w') as out:
        with open(os.path.join(INDIR, infile), 'r') as infile:
            for l in infile:
                out.write(
                    re.sub(r'^(map_\w+) (?:.*/)?([^/]+)$', r'\1 \2', l)
                )


def build_mesh(infile, outfile):
    """Rewrite the mesh file to use simple mtllib paths."""
    with open(os.path.join(OUTDIR, outfile), 'w') as out:
        with open(os.path.join(INDIR, infile), 'r') as infile:
            for l in infile:
                out.write(
                    re.sub(r'^mtllib (\S+?)_\d+\.mtl$', r'mtllib \1.mtl', l)
                )


def get_frames(basename):
    for f in os.listdir(INDIR):
        if f.startswith(basename + '_') and f.endswith('.obj'):
            yield f


def write_vertex_animation(basename, frames):
    with open(os.path.join(OUTDIR, basename + '.anm'), 'w') as out:
        for i, f in enumerate(frames):
            out.write('frame %d\n' % (i + 1))
            with open(f) as inframe:
                for l in inframe:
                    if re.match(r'^(vn? )', l):
                        out.write(l)


def build_frames(basename):
    frames = sorted(get_frames(basename))
    for f in frames:
        build_mesh(f, f)


def build_animation(basename):
    if os.path.exists(os.path.join(INDIR, basename + '_000001.obj')):
        build_material(basename + '_000001.mtl', basename + '.mtl')
        build_frames(basename)
    else:
        build_material(basename + '.mtl', basename + '.mtl')
        build_mesh(basename + '.obj', basename + '.obj')


if __name__ == '__main__':
    import sys
build_animation(sys.argv[1])
