from __future__ import division

import math
import pyglet
from pyglet import gl
from pyglet.window import key
import pyglet.resource

from scenegraph.objloader import ModelLoader, load_texture

from scenegraph.scenegraph import ModelNode, Scene, Camera, ShaderGroup, v3, RenderPass
from scenegraph.shaderlib import textured_specular
from scenegraph.particles import ParticleSystemNode
from scenegraph.lighting import DepthOnlyPass, LightingPass, Light, CompositePass


# Set up pyglet resource loader
pyglet.resource.path += [
    'assets/textures/'
]
pyglet.resource.reindex()


import particle_effects


FPS = 60
WIDTH = 800
HEIGHT = 600


mesh = None
wiz = None
wiz2 = None
particles = None


loader = ModelLoader()

lighting = LightingPass(
    ambient=(0.05, 0.05, 0.05, 1.0)
)
scene = Scene([
    lighting,
    CompositePass(lighting),
    RenderPass(transparency=True)
])


scene.add(Light(
    pos=(-1000, 1000, -1000, 0),
    colour=(1.0, 1.0, 1.0, 1.0),
    intensity=0.5,
    falloff=0
))

#scene.add(Light(
#    pos=(0, 0, 0, 0),
#    colour=(1.0, 0.4, 0.4, 1.0),
#    intensity=3000.0,
#    falloff=2.0
#))

light = Light(
    pos=(2, 5, 5, 0),
    colour=(0, 1, 0, 0),
    intensity=8000.0,
    falloff=4.0
)
scene.add(light)

c = Camera(
    pos=v3((20, 4, -27)),
    look_at=v3((0, 4, 0))
)


def load():
    global mesh, wiz, wiz2, particles, test_pg
    parent = None #ShaderGroup(textured_specular)
#    parent.uniformf('light_pos', *LIGHT_POS)

    model = loader.load_animation('wizard')

    wiz = ModelNode(model,
        pos=v3((-15, 0, 0)),
        rotation=(90, 0, 1, 0),
        group=parent
    )
    scene.add(wiz)

    wiz2 = ModelNode(model,
        pos=v3((15, 0, 0)),
        rotation=(-90, 0, 1, 0),
        group=parent
    )
    scene.add(wiz2)

    forest_model = loader.load_model('forest')
    forest = ModelNode(forest_model, rotation=(180, 0, 1, 0), group=parent)
    scene.add(forest)

    particles = ParticleSystemNode()
    scene.add(particles)
    from particle_effects import magic_missile, Particle, StaticEmitter
    test_pg = particles.create_group((), texture=magic_missile)

    e = StaticEmitter(
        template=Particle(
            rotation=(0, 0, 6),
            color=(1, 1, 1, 1),
            velocity=(-15, 0, 0),
            size=(0.5, 0.5, 0.5),
        )
    )
    e.emit(1, test_pg)



zpos = 6
tx, ty = 0, 2


# Pixels the mouse has to move to move the camera 1 degree
MOUSE_SPEED = 4.0

# Theta is left-right rotation around the scene
theta = 0
MAX_THETA = 80
MIN_THETA  = -80

# Phi is how high we are looking down on the scene
phi = 10
MIN_PHI = 2
MAX_PHI = 30

# Angle of the light
a = 0


def update_camera():
    sinphi = math.sin(math.radians(phi))
    cosphi = math.cos(math.radians(phi))
    sintheta = math.sin(math.radians(theta))
    costheta = math.cos(math.radians(theta))

    c.pos = v3(cosphi * sintheta, sinphi, -cosphi * costheta) * (zpos + 24)


update_camera()


def update(dt):
    global zpos,  a

    # Control camera
    zpos *= 0.8 ** dt
    update_camera()

    # Bouncing light
    a += 1.0 * dt
    light.pos = (12 * math.sin(a), 4, 0, 0)
    if len(test_pg):
        p = next(iter(test_pg))
        p.position = light.pos[:3]

    scene.update(dt)


def on_draw():
    gl.glClearColor(0, 0, 0, 0)
    gl.glClear(gl.GL_COLOR_BUFFER_BIT | gl.GL_DEPTH_BUFFER_BIT)

    gl.glDisable(gl.GL_CULL_FACE)

    gl.glEnable(gl.GL_BLEND)
    gl.glBlendFunc(gl.GL_SRC_ALPHA, gl.GL_ONE_MINUS_SRC_ALPHA)
    gl.glEnable(gl.GL_ALPHA_TEST)
    gl.glAlphaFunc(gl.GL_GREATER, 0.9)

    scene.render(c)


attacks = []


def on_key_press(symbol, modifiers):
    if symbol == key.X:
        attacks.append(particle_effects.cast_fire(
            particles,
            wiz2.pos
        ))
    elif symbol == key.Z:
        attacks.append(particle_effects.cast_magic_missile(
            particles,
            wiz2.pos,
            wiz.pos
        ))


def on_mouse_motion(x, y, dx, dy):
    global theta, phi
    theta = max(MIN_THETA, min(MAX_THETA, theta + dx / MOUSE_SPEED))
    phi = max(MIN_PHI, min(MAX_PHI, phi + dy / MOUSE_SPEED))


if __name__ == '__main__':
    window = pyglet.window.Window(
        width=WIDTH,
        height=HEIGHT
    )
    load()

    window.event(on_draw)
    window.event(on_key_press)
    window.event(on_mouse_motion)
    pyglet.clock.schedule_interval(update, 1.0 / FPS)
    pyglet.app.run()
