import lepton
from lepton.emitter import StaticEmitter, PerParticleEmitter
from lepton import Particle
from lepton import domain
from lepton import controller
from lepton.domain import Vec3

from scenegraph.objloader import load_texture


fireball = load_texture('fire.png')
magic_missile = load_texture('magic-missile.png')
spark = load_texture('spark.png')


def cast_fire(particles, pos):
    particle_group = particles.create_group(texture=fireball, controllers=[
        controller.Movement(),
        controller.Fader(
            start_alpha=0.0,
            fade_in_start=0.0,
            fade_in_end=0.2,
            max_alpha=1.0,
            fade_out_start=1.0,
            fade_out_end=1.5,
            end_alpha=0.0
        ),
        controller.Lifetime(1.5),
        controller.Growth(1.5)
    ])

    pos = pos + (-4, 4, 0)
    pos_domain = domain.Cone(
        pos,
        pos + (3, 0, 0),
        0.5
    )

    template = Particle(
        color=(1, 1, 1),
        velocity=(-25, 0, 0)
    )

    emitter = StaticEmitter(
        position=pos_domain,
        velocity=domain.Sphere((-25, 0, 0), 4),
        size=domain.Line((0.1, 0.1, 0.1), (1.0, 1.0, 1.0)),
        template=template,
        #deviation=deviation,
        rate=200,
        time_to_live=1,
    )

    particle_group.bind_controller(emitter)
    return particle_group


def cast_magic_missile(particles, pos, target, num=1):
    pos = pos + (-4, 4, 0)
    target = target + (4, 4, 0)

    missile_group = particles.create_group(texture=magic_missile, controllers=[
        controller.Movement(),
        controller.Lifetime(0.8),
        controller.Magnet(
            domain.Sphere(target, 1),
            70,
            0
        ),
        controller.Fader(
            start_alpha=0.0,
            fade_in_start=0.0,
            fade_in_end=0.2,
            max_alpha=1.0,
        ),
        controller.Growth(5, 0.96)
    ])

    pos_domain = domain.Cone(
        pos,
        pos + (3, 0, 0),
        0.5
    )

    emitter = StaticEmitter(
        template=Particle(
            rotation=(0, 0, 6),
            color=(1, 1, 1, 1),
            velocity=(-15, 0, 0),
            size=(0.1, 0.1, 0.1),
        ),
        position=pos_domain,
        velocity=domain.Disc((-10, 0, 0), (1, 0, 0), 7, 5),
    )

    emitter.emit(num, missile_group)

    spark_group = particles.create_group(texture=spark, controllers=[
        controller.Movement(),
        controller.Gravity((0, 1, 0)),
        controller.Lifetime(0.8),
        controller.Growth(0.1),
        controller.Fader(
            start_alpha=0.0,
            fade_in_start=0.0,
            fade_in_end=0.035,
            max_alpha=1.0,
            fade_out_start=0.2,
            fade_out_end=0.8,
            end_alpha=0.0
        ),
    ])

    per_particle_emitter = PerParticleEmitter(
        missile_group,
        template=Particle(
            size=(0.2, 0.2, 0.2),
            color=(0.6, 1.0, 0.8, 1.0),
        ),
        deviation=Particle(
            position=(0.1, 0.1, 0.1),
#            color=(0.1, 0.1, 0.1, 0),
        ),
        rate=50,
        rotation=domain.Line((0, 0, 0), (0, 0, 6)),
        up=domain.Line((0, 0, 0), (0, 0, 1))
    )

    spark_group.bind_controller(per_particle_emitter)
    return (missile_group, spark_group)

